import { createId } from '@paralleldrive/cuid2';
import { createFloatingActions } from 'svelte-floating-ui';
import { flip } from 'svelte-floating-ui/dom';
import { get } from 'svelte/store';
export function isElementInPopover(srcElement) {
    if (!srcElement)
        return false;
    return !!srcElement.closest('[data-popover-id]');
}
const defaultOptions = {
    strategy: 'absolute',
    placement: 'bottom-start',
    middleware: [flip()],
};
export function createPopover(isOpen, options) {
    options = { ...defaultOptions, ...options };
    const id = createId();
    function customTrigger(node) {
        function nodeClick() {
            isOpen.update((value) => !value);
        }
        function clickOutside(e) {
            const clickedNode = e.target.closest(`[data-popover-id="${id}"]`);
            let clickedNodeId;
            if (clickedNode) {
                clickedNodeId = clickedNode.dataset.popoverId;
            }
            if (node && clickedNodeId !== id && !e.defaultPrevented) {
                isOpen.set(false);
            }
        }
        function keyUp(event) {
            if (get(isOpen) && event.key === 'Escape') {
                isOpen.set(false);
            }
        }
        node.dataset.popoverId = id;
        node.addEventListener('click', nodeClick);
        document.addEventListener('click', clickOutside);
        document.addEventListener('keyup', keyUp);
        return {
            destroy: () => {
                node.removeEventListener('click', nodeClick);
                document.removeEventListener('click', clickOutside);
                document.removeEventListener('keyup', keyUp);
            },
        };
    }
    function customContent(node) {
        node.dataset.popoverId = id;
        node.focus();
        return {
            destroy: () => { },
        };
    }
    const [floatingTrigger, floatingContent, update] = createFloatingActions(options);
    const triggerActions = [customTrigger, floatingTrigger];
    function triggerAction(node) {
        const destructors = triggerActions.map((action) => action(node));
        return {
            destroy() {
                destructors.forEach((destructor) => {
                    if (typeof destructor?.destroy === 'function')
                        destructor.destroy();
                });
            },
        };
    }
    const contentActions = [floatingContent, customContent];
    function contentAction(node) {
        contentActions.map((action) => action(node));
        return {
            destroy() { },
        };
    }
    return [triggerAction, contentAction, update];
}
